#! /usr/bin/env python
#
#    forest/server.py sits on the socket and acts as doorman
#    Copyright (C) 2002  Michael Urman
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import forest
import linux_getpeereid_socket as socket
import os, sys, pwd, grp, select, time

class server:

    def __init__(self):
        """Initialize the server.  Attempt to allocate the server socket.  If it
        exists, attempt to verify it is alive.  If not, wipe and attempt to
        allocate teh socket."""

        self.address = forest.socket
        serv = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        try:
            serv.bind(self.address)
        except socket.error bind_error:
            # socket exists; may be in use.  Test
            try:
                serv.connect(self.address)
            except socket.error, connect_error:
                # connection failed; assume stale
                print>>sys.stderr, "Found stale server socket %s.  Removing." % self.address
                os.unlink(self.address)
                try:
                    serv.bind(self.address)
                except socket.error, new_error:
                    print>>sys.stderr, "Couldn't create server socket.  Errors received:"
                    print>>sys.stderr, " - first attempt:", bind_error
                    print>>sys.stderr, " - check for stale:", connect_error
                    print>>sys.stderr, " - second attempt:", new_error
                    sys.exit(1)
            else:
                # connection succeeded; give up
                serv.shutdown(2)
                serv.close()
                print>>sys.stderr, "Server already running on %s.  Exiting." % self.address
                sys.exit(1)

        # serv is a bound server socket
        os.chmod(self.address, 0777) # allow anyone to connect
        serv.listen(socket.SOMAXCONN)
        self.server_socket = serv
        self.clients = { serv.fileno() : serv }

    def __del__(self):
        """__del__(self) -> None

        Attempt some minimum cleanup:
            - close all sockets
            - remove server socket"""

        self.cleanclients()
        try:
            self.server_socket.shutdown(2)
            self.server_socket.close()
        finally:
            if os.path.exists(self.address):
                os.unlink(self.address)

    def cleanclients(self):
        """cleanclients(self) -> None
        
        Close and delete all clients"""

        for client in self.clients.values():
            try:
                client.shutdown(2)
                client.close()
            except socket.error:
                pass

        del self.clients
        self.clients = []

    def run(self):
        """run(self) -> None

        Run the server, servicing everything."""

        try:
            while 1:
                # wait for something
                rlist = self.clients.keys()
                wlist = self.pending_writes.keys()
                rlist, wlist, xlist = select.select(rlist, wlist, rlist)

                for errfd in xlist:
                    try:
                        sock = self.clients[errfd]
                        sock.shutdown(2)
                        sock.close()
                        del self.clients[errfd]
                    except (KeyError, socket.error):
                        pass

                for ready in rlist:
                    try:
                        sock = self.fd_to_sock[ready]

                        if sock == self.server_socket:
                            # server socket implies a connecting client
                            self.new_client()
                        else:
                            # a ready client implies [part of] a message
                            messages = self.get_message(sock)
                            for message in messages:
                                self.handle(sock, message)
                    except KeyError:
                        pass    # xlist caused removal

        finally:
            self.cleanclients()

    def new_client(self):
        pass
